"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const fs = require("fs");
const path = require("path");
/**
 * Returns the name of the log file
 */
function createLogFile(app, tag) {
    return __awaiter(this, void 0, void 0, function* () {
        yield createApplicationLogPath(app);
        const logfile = getLogFilename(app, tag);
        return logfile;
    });
}
exports.createLogFile = createLogFile;
const logBasePath = path.resolve(__dirname, '..', '..', '..', '..', 'logs');
function getLogFilename(app, ref) {
    const now = new Date();
    const date = now.toISOString().slice(0, 10);
    const time = now
        .toISOString()
        .slice(11, 19)
        .replace(/:/g, '-');
    const filename = `${date}_${time}__${ref}.json`.split('/').join('_');
    const logPath = path
        .resolve(logBasePath, app.id.toString(), filename)
        .replace('refs/heads/', '')
        .replace('refs/tags/', '');
    log.debug(`Using log file: ${logPath}`);
    return logPath;
}
function createApplicationLogPath(application) {
    return __awaiter(this, void 0, void 0, function* () {
        return mkdirAsync(path.resolve(logBasePath, application.id.toString()));
    });
}
function handleBuildStream(stream, record) {
    const buildResponses = [];
    const promise = new Promise((resolve, reject) => {
        stream.on('data', (data) => {
            const msg = data.toString();
            const output = tryParse(msg);
            if (!Array.isArray(output)) {
                record([output]);
                return;
            }
            buildResponses.push(...output);
            const toLog = output
                .map(ev => ev.stream || ev.errorDetail)
                .filter(ev => {
                if (!ev)
                    return false;
                return !!(ev.message || ev || '').trim();
            });
            record(toLog);
        });
        stream.on('end', () => {
            const hasErrors = buildResponses.some(res => res.hasOwnProperty('errorDetail'));
            if (hasErrors)
                return reject(buildResponses);
            resolve(buildResponses);
        });
    });
    return promise;
}
exports.handleBuildStream = handleBuildStream;
function tryParse(text) {
    try {
        const json = JSON.parse(text.trim());
        return [json];
    }
    catch (_) {
        try {
            const split = text.trim().split('\n');
            return split.map(splitText => JSON.parse(splitText));
        }
        catch (__) {
            return text;
        }
    }
}
function mkdirAsync(folder) {
    return new Promise((resolve, reject) => {
        fs.mkdir(folder, err => {
            if (err) {
                const msg = (err.message || err);
                if (msg.indexOf('EEXIST') > -1) {
                    return resolve();
                }
                return reject(err);
            }
            resolve();
        });
    });
}
//# sourceMappingURL=log.js.map