"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const types_1 = require("../types");
const child_process_1 = require("child_process");
const path_1 = require("../../git/path");
function getCommands(task, stage) {
    if (!task) {
        return [];
    }
    const commands = task[stage];
    if (!Array.isArray(commands)) {
        return [];
    }
    return commands;
}
exports.getCommands = getCommands;
/**
 * Environment variables passed into each step:
 * CONCIERGE_REPO: Git repository url
 * CONCIERGE_APP: Application name
 * CONCIERGE_LABEL: Application label (Used to generate image tag)
 * CONCIERGE_REF: Git ref - Branch or tag name
 * CONCIERGE_SHA: Git SHA
 * CONCIERGE_IMAGE_TAG?: Docker tag
 * CONCIERGE_IMAGE_ID?: Docker image ID
 * CONCIERGE_STATUS ("Successful", "Failed")
 */
function executeTask(opts, command) {
    const env = {
        CONCIERGE_REPO: opts.app.repository,
        CONCIERGE_APP: opts.app.name,
        CONCIERGE_LABEL: opts.app.label,
        CONCIERGE_REF: opts.ref,
        CONCIERGE_SHA: opts.sha,
        CONCIERGE_IMAGE_TAG: opts.imageTag || '',
        CONCIERGE_IMAGE_ID: opts.imageId || '',
        CONCIERGE_STATUS: types_1.State[opts.state]
    };
    const cwd = path_1.default(opts.app);
    return new Promise(resolve => {
        const [cmd, ...args] = command.split(' ').filter(cmd => !!cmd);
        const proc = child_process_1.spawn(cmd, args, {
            cwd,
            env,
            shell: true
        });
        const output = [];
        proc.stdout.on('data', data => {
            output.push(...data.toString().split('\n'));
        });
        proc.stderr.on('error', (err) => output.push(err.message || err));
        proc.on('close', code => resolve({ code, output }));
    });
}
exports.executeTask = executeTask;
//# sourceMappingURL=task.js.map