"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const db = require("./db");
const queue_1 = require("./monitor/queue");
const handler = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const id = Number(req.params.id);
    const { tag, sha, ref } = req.query;
    const app = yield db.one(id);
    if (!app) {
        res.status(400).json({ message: `Application '${id}' does not exist` });
        return;
    }
    try {
        /**
         * This will fail if:
         * - This is no available host
         * - It fails to pack the repo tar ball due to a deployment of that image already taking place
         *
         * It will not await the entire build
         */
        queue_1.queue.add(app, {
            age: new Date(),
            ref,
            sha,
            seen: new Date()
        });
        res.json({ message: `Added image '${tag}' to build queue` });
    }
    catch (ex) {
        log.error(ex.message || ex);
        if (ex.stack) {
            log.debug(ex.stack);
        }
        res.status(500).json({ message: ex.message || ex });
    }
});
exports.default = handler;
//# sourceMappingURL=build.js.map