"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const pack_1 = require("../git/pack");
const path_1 = require("../git/path");
const path_2 = require("path");
const fs_1 = require("fs");
const tasks_1 = require("../tasks");
function checkout(app, sha, attempt = 1) {
    return __awaiter(this, void 0, void 0, function* () {
        /**
         * TODO
         * - Switch over to shallow clones (clone with --depth 1)
         * - Concierge maintains a long-lived repository folder for tracking remotes
         * - Hosts will always clone: git clone -b [SHA] --single-branch --depth 1 [repo] [temp-location]
         */
        // TODO: This should use a specific host
        try {
            const stream = yield pack_1.default(app, sha);
            const task = yield getTaskFile(app);
            return { task, stream };
        }
        catch (ex) {
            if (attempt === 5) {
                throw ex;
            }
            if (ex.code !== 'E_REPOBUSY') {
                throw ex;
            }
            // We'll try every 250ms up to a maximum of 5 tries
            return new Promise(resolve => {
                setTimeout(() => {
                    resolve(checkout(app, sha, ++attempt));
                }, 250);
            });
        }
    });
}
exports.checkout = checkout;
function getTaskFile(app) {
    return __awaiter(this, void 0, void 0, function* () {
        const base = path_1.default(app);
        const files = ['concierge.yml', 'concierge.yaml', 'concierge.json'];
        for (const file of files) {
            const content = yield readFileAsync(path_2.join(base, file));
            if (!content) {
                continue;
            }
            const task = tasks_1.parseTaskFile(content, path_2.extname(file));
            if (task) {
                return task;
            }
        }
        return null;
    });
}
function readFileAsync(file) {
    return new Promise(resolve => {
        fs_1.readFile(file, (err, data) => {
            if (err) {
                return resolve(null);
            }
            resolve(data.toString());
        });
    });
}
//# sourceMappingURL=checkout.js.map