"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const validate_1 = require("./validate");
const clone_1 = require("../git/clone");
const path = require("path");
const fs = require("fs");
const db = require("../../data");
const monitor_1 = require("./monitor");
const util_1 = require("util");
const handler = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { repository, name, label, dockerfile, autoBuild = false, credentialsId = 0 } = req.body;
    const id = yield getNextId();
    const body = { id, repository, name, label, dockerfile, credentialsId, autoBuild };
    const errors = validate_1.default(body);
    if (errors.length) {
        const message = errors.join(', ');
        res.status(400).json({ message });
        return;
    }
    try {
        yield db.applications().insert(body);
        const app = Object.assign({}, body);
        yield clone_1.default(app);
        res.json(app);
        // Once the application is ready and cloned, we will begin to track it
        monitor_1.poll();
    }
    catch (ex) {
        res.status(500);
        res.json({ message: ex.message || ex });
    }
});
function getNextId() {
    return __awaiter(this, void 0, void 0, function* () {
        const apps = yield db.applications().select('id');
        const dirs = yield getRepoDirs();
        const ids = dirs.map(dir => Number(dir)).filter(id => !isNaN(id));
        const max = Math.max(...ids, ...apps.map(app => app.id));
        if (max < 1) {
            return 1;
        }
        return max + 1;
    });
}
function getRepoDirs() {
    return __awaiter(this, void 0, void 0, function* () {
        const repoPath = path.resolve(__dirname, '../../..', 'repositories');
        try {
            const dirs = yield readdir(repoPath);
            return dirs;
        }
        catch (ex) {
            if (ex.code !== 'ENOENT') {
                throw ex;
            }
            yield mkdir(repoPath);
            const dirs = yield readdir(repoPath);
            return dirs;
        }
    });
}
const readdir = util_1.promisify(fs.readdir);
const mkdir = util_1.promisify(fs.mkdir);
exports.default = handler;
//# sourceMappingURL=create.js.map