"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const db = require("../../data");
function one(id) {
    return __awaiter(this, void 0, void 0, function* () {
        const application = (yield db
            .applications()
            .select(`${db.APPLICATIONS}.*`, `${db.CREDENTIALS}.name as credentialsName`)
            .where(`${db.APPLICATIONS}.id`, id)
            .leftJoin(db.CREDENTIALS, `${db.CREDENTIALS}.id`, `${db.APPLICATIONS}.credentialsId`)
            .first());
        if (!application) {
            return;
        }
        application.credentialsName = application.credentialsName || 'N/A';
        return application;
    });
}
exports.one = one;
function all() {
    return __awaiter(this, void 0, void 0, function* () {
        const applications = yield db
            .applications()
            .select(`${db.APPLICATIONS}.*`, `${db.CREDENTIALS}.name as credentialsName`)
            .leftJoin(db.CREDENTIALS, `${db.CREDENTIALS}.id`, `${db.APPLICATIONS}.credentialsId`)
            .orderBy(`${db.APPLICATIONS}.id`, 'asc');
        for (const app of applications) {
            app.credentialsName = app.credentialsName || 'N/A';
            app.autoBuild = !!app.autoBuild;
        }
        return applications;
    });
}
exports.all = all;
function remove(id) {
    return __awaiter(this, void 0, void 0, function* () {
        yield db
            .applicationRemotes()
            .delete()
            .where('applicationId', id);
        yield db
            .applications()
            .delete()
            .where('id', id);
    });
}
exports.remove = remove;
function getRemotes(applicationId) {
    return __awaiter(this, void 0, void 0, function* () {
        const remotes = yield db
            .applicationRemotes()
            .select()
            .where('applicationId', applicationId)
            .orderBy('seen', 'asc');
        return remotes;
    });
}
exports.getRemotes = getRemotes;
function removeRemote(applicationId, remote) {
    return __awaiter(this, void 0, void 0, function* () {
        const rowsDeleted = yield db
            .applicationRemotes()
            .delete()
            .where('applicationId', applicationId)
            .andWhere('remote', remote);
        if (rowsDeleted !== 1) {
            throw new Error('Failed to remove ApplicationRemote: Row not found');
        }
    });
}
exports.removeRemote = removeRemote;
function updateRemote(applicationId, ref, props) {
    return __awaiter(this, void 0, void 0, function* () {
        return db
            .applicationRemotes()
            .update(props)
            .where('applicationId', applicationId)
            .andWhere('remote', ref);
    });
}
exports.updateRemote = updateRemote;
function insertRemote(remote) {
    return __awaiter(this, void 0, void 0, function* () {
        return db.applicationRemotes().insert(remote);
    });
}
exports.insertRemote = insertRemote;
function getRemote(applicationId, ref) {
    return __awaiter(this, void 0, void 0, function* () {
        const remote = yield db
            .applicationRemotes()
            .select()
            .where('applicationId', applicationId)
            .andWhere('remote', ref)
            .first();
        return remote;
    });
}
exports.getRemote = getRemote;
function getAllRemotes() {
    return __awaiter(this, void 0, void 0, function* () {
        const remotes = yield db.applicationRemotes().select();
        return remotes;
    });
}
exports.getAllRemotes = getAllRemotes;
//# sourceMappingURL=db.js.map