"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const db = require("../db");
const monitor_1 = require("./monitor");
const db_1 = require("../../configuration/db");
function monitorAll() {
    return __awaiter(this, void 0, void 0, function* () {
        yield monitor.init();
    });
}
exports.default = monitorAll;
class MonitorAll {
    constructor() {
        this.monitors = [];
        this.initialised = false;
        this.init = () => __awaiter(this, void 0, void 0, function* () {
            const applications = yield db.all();
            this.monitors = applications.map(app => new monitor_1.RemoteMonitor(app));
            const promises = this.monitors.map(monitor => monitor.initialise(false));
            yield Promise.all(promises);
            this.initialised = true;
            const { gitPollingIntervalSecs } = yield db_1.getConfig();
            setTimeout(() => this.poll(), gitPollingIntervalSecs * 1000);
        });
        this.poll = () => __awaiter(this, void 0, void 0, function* () {
            try {
                const apps = yield db.all();
                for (const app of apps) {
                    const monitor = this.monitors.find(monitor => monitor.app.id === app.id);
                    if (monitor) {
                        continue;
                    }
                    const monitoredApp = new monitor_1.RemoteMonitor(app);
                    yield monitoredApp.initialise(true);
                    this.monitors.push(monitoredApp);
                }
            }
            catch (_a) {
                /** Intentional NOOP */
            }
        });
    }
}
const monitor = new MonitorAll();
function poll() {
    return monitor.poll();
}
exports.poll = poll;
//# sourceMappingURL=index.js.map