"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const db = require("../../data");
const os = require("os");
const defaultConfig = {
    name: os.hostname() || 'development',
    conciergePort: 3141,
    debug: 0,
    dockerRegistry: undefined,
    proxyHostname: undefined,
    registryCredentials: undefined,
    statsBinSize: 60,
    statsRetentionDays: 3,
    maxConcurrentBuilds: 2,
    gitPollingIntervalSecs: 5
};
function getConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        const result = yield db
            .configurations()
            .select()
            .where('id', 1)
            .first();
        const config = JSON.parse(result.config);
        return Object.assign(Object.assign({}, defaultConfig), config);
    });
}
exports.getConfig = getConfig;
function setConfig(config) {
    return __awaiter(this, void 0, void 0, function* () {
        const original = yield getConfig();
        const next = Object.assign(Object.assign(Object.assign(Object.assign({}, defaultConfig), original), config), { conciergePort: original.conciergePort });
        const keys = Object.keys(config);
        const badKeys = [];
        for (const key of keys) {
            const validator = validate[key];
            if (!validator) {
                continue;
            }
            if (!validator(config[key])) {
                badKeys.push(key);
            }
        }
        if (badKeys.length > 0) {
            const message = badKeys.join(', ');
            throw new Error(`Invalid configuration, keys have invalid values: ${message}`);
        }
        yield db
            .configurations()
            .update('config', JSON.stringify(next))
            .where('id', 1);
        return next;
    });
}
exports.setConfig = setConfig;
const validate = {
    debug: val => isNum(val),
    gitPollingIntervalSecs: val => isNum(val, 0),
    statsBinSize: val => isNum(val, 0),
    maxConcurrentBuilds: val => isNum(val, 0),
    statsRetentionDays: val => isNum(val, 0)
};
function isNum(value, greaterThan) {
    const num = Number(value);
    if (isNaN(num)) {
        return false;
    }
    return greaterThan === undefined ? true : num > greaterThan;
}
//# sourceMappingURL=db.js.map