"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const DockerClient = require("dockerode");
const process = require("process");
const db_1 = require("../configuration/db");
const db_2 = require("../credentials/db");
function getDockerClient(host, timeout) {
    return __awaiter(this, void 0, void 0, function* () {
        // If no hostname is provided, try and use the local unix socket
        if (!host.hostname) {
            const socketPath = process.platform === 'win32' ? '//./pipe/docker_engine' : '/var/run/docker.sock';
            const dockerClient = new DockerClient({
                socketPath,
                timeout: timeout || 0
            });
            return dockerClient;
        }
        const dockerClient = new DockerClient({
            host: host.hostname,
            port: host.dockerPort || 2375,
            protocol: 'http',
            timeout: timeout || 0
        });
        return dockerClient;
    });
}
exports.getDockerClient = getDockerClient;
function getAuthConfig() {
    return __awaiter(this, void 0, void 0, function* () {
        const cfg = yield db_1.getConfig();
        if (!cfg.registryCredentials) {
            return {
                auth: ''
            };
        }
        const creds = yield db_2.one(cfg.registryCredentials);
        const authConfig = creds.username
            ? { username: creds.username, password: creds.key }
            : { key: creds.key };
        return Object.assign({ auth: '', serveraddress: cfg.dockerRegistry }, authConfig);
    });
}
exports.getAuthConfig = getAuthConfig;
//# sourceMappingURL=index.js.map