"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cmd_1 = require("./cmd");
const path_1 = require("./path");
const tar = require("tar-fs");
const sentinal = {};
/**
 * ref format:
 * - refs/heads/branch-name
 * - refs/tags/tag-name
 */
function refToStream(application, sha) {
    return __awaiter(this, void 0, void 0, function* () {
        const id = Number(application.id);
        const isBusy = !!sentinal[id];
        if (isBusy) {
            const err = new Error(`Unable to pack: Application folder is busy`);
            err.code = 'E_REPOBUSY';
            throw err;
        }
        sentinal[id] = true;
        const workDir = path_1.default(application);
        try {
            yield cmd_1.default(application, workDir, `git fetch origin ${sha}`);
            yield cmd_1.default(application, workDir, `git checkout ${sha}`);
        }
        catch (ex) {
            sentinal[id] = false;
            throw new Error(`Failed to pack ${sha}: ${ex.message || ex}`);
        }
        const stream = tar.pack(workDir);
        const resetSentinal = () => {
            sentinal[id] = false;
        };
        stream.on('close', resetSentinal);
        stream.on('end', resetSentinal);
        stream.on('error', resetSentinal);
        return stream;
    });
}
exports.default = refToStream;
//# sourceMappingURL=pack.js.map