"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const cmd_1 = require("./cmd");
const path_1 = require("./path");
function getRemoteTags(application, branchesOnly = false) {
    return __awaiter(this, void 0, void 0, function* () {
        const workDir = path_1.default(application);
        const args = branchesOnly ? `--heads` : `--tags --heads`;
        const command = `git ls-remote ${args}`;
        const result = yield cmd_1.default(application, workDir, command);
        const ages = yield getRemoteAges(application);
        const getAge = (ref) => {
            const refAge = ages.find(age => age.ref === ref);
            if (refAge) {
                return refAge.age;
            }
            return undefined;
        };
        const tags = result
            .split('\n')
            .filter(line => !!line.trim())
            .filter(line => line.indexOf('refs/') > 0)
            .map(raw => {
            const split = raw.split('refs/');
            const rawRef = split[1].trim();
            const sha = split[0].trim();
            const ref = rawRef.replace('heads/', '').replace('tags/', '');
            const type = rawRef.startsWith('heads/') ? 'branch' : 'tag';
            const age = getAge(ref);
            return { type, ref, sha, age };
        })
            .filter(ref => !!ref.ref && !!ref.age)
            .filter(ref => !ref.ref.endsWith('^{}'));
        return tags;
    });
}
exports.default = getRemoteTags;
function getRemoteAges(app) {
    return __awaiter(this, void 0, void 0, function* () {
        const cwd = path_1.default(app);
        // We need to update our local version of remotes before using the for-each-ref command
        yield cmd_1.default(app, cwd, `git remote update origin --prune`);
        const command = `git`;
        const args = [
            'for-each-ref',
            '--sort=-committerdate',
            '--format={"age":"%(committerdate:iso)", "ref":"%(refname)", "author":"%(authorname)" }',
            'refs/remotes'
        ];
        const data = yield cmd_1.default(app, cwd, command, args);
        const refs = data
            .split('\n')
            .filter(line => !!line.trim() && line.indexOf('refs/remotes/origin/HEAD') === -1)
            .map(line => JSON.parse(line))
            .map(ref => (Object.assign(Object.assign({}, ref), { age: new Date(ref.age), ref: ref.ref.replace('refs/remotes/origin/', '') })));
        return refs;
    });
}
// git for-each-ref --sort=-committerdate --format="\"{\\\"date\\\":\\\"%(committerdate:iso)\\\", \\\"ref\\\":\\\"%(refname)\\\", \\\"author\\\":\\\"%(authorname)\\\" }\"" refs/remotes
// git for-each-ref --sort=-committerdate --format="%(committerdate:iso) %refname %authorname" refs\remotes
//# sourceMappingURL=tags.js.map