"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const docker_1 = require("../docker");
const db = require("../hosts/db");
const handler = (req, res) => __awaiter(void 0, void 0, void 0, function* () {
    const { name, image, envs, ports, volumes, links } = req.body;
    const exposedPorts = ports.filter(port => port.expose).reduce((prev, curr) => {
        const key = `${curr.port}/${curr.type}`;
        prev[key] = {};
        return prev;
    }, {});
    const portBindings = ports.filter(port => port.expose).reduce((prev, curr) => {
        const key = `${curr.port}/${curr.type}`;
        const hostCfg = {};
        // If a hostPort is specified, pass the option through to Docker
        if (curr.hostPort) {
            hostCfg.HostPort = curr.hostPort;
        }
        prev[key] = [hostCfg];
        return prev;
    }, {});
    const binds = volumes.filter(vol => !!vol.hostPath).map(vol => `${vol.hostPath}:${vol.path}`);
    const containerLinks = links.map(link => `${link.containerName}:${link.alias}`);
    const options = {
        name,
        Image: image,
        Env: envs.map(env => `${env.key}=${env.value}`),
        ExposedPorts: exposedPorts,
        HostConfig: {
            Binds: binds,
            PortBindings: portBindings,
            RestartPolicy: {
                Name: 'on-failure',
                MaximumRetryCount: 3
            },
            Links: containerLinks
        }
    };
    if (name.length === 0) {
        delete options.name;
    }
    const host = yield getAvailableHost();
    try {
        const container = yield host.client.createContainer(options);
        yield container.start();
        res.json({
            id: container.id,
            host: host.host.hostname
        });
    }
    catch (ex) {
        res.status(500).json({ message: ex.message || ex });
    }
});
function getAvailableHost() {
    return __awaiter(this, void 0, void 0, function* () {
        const hosts = yield db.getAll();
        if (hosts.length === 0) {
            throw new Error(`No hosts exists`);
        }
        const loads = yield Promise.all(hosts.map(getHostLoads));
        loads.sort((left, right) => (left > right ? 1 : -1));
        const leastLoaded = loads[0];
        return leastLoaded;
    });
}
function getHostLoads(host) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = docker_1.default(host);
        const containers = yield client.listContainers();
        const percentage = containers.length / host.capacity;
        return {
            host,
            client,
            percentage,
            containers: containers.length,
            capacity: host.capacity
        };
    });
}
exports.default = handler;
//# sourceMappingURL=run.js.map