"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const getHosts = require("../hosts/db");
const containers_1 = require("./containers");
const emitter = require("./emitter");
const docker_1 = require("../docker");
const hostState = {};
function watchHosts() {
    return __awaiter(this, void 0, void 0, function* () {
        const hosts = yield getHosts.getAll();
        for (const host of hosts) {
            const hostId = host.id;
            const isWatching = hostState[hostId];
            if (isWatching) {
                continue;
            }
            const client = docker_1.default(host);
            const hostname = host.hostname || host.vanityHostname;
            client.getEvents({}, (err, stream) => {
                if (err || !stream) {
                    log.error(`[${hostname}] Failed to monitor host: ${err}`);
                    return;
                }
                log.info(`[${hostname}] Monitoring host`);
                hostState[hostId] = true;
                stream.on('data', (event) => handleHostEvent(host, event));
            });
        }
    });
}
exports.default = watchHosts;
function handleHostEvent(host, rawEvent) {
    const events = parseEvent(rawEvent);
    if (!events) {
        return;
    }
    for (const event of events) {
        const id = (event.id || 'Host').slice(0, 10);
        const status = (event.status || 'unknown').toLowerCase();
        log.info(`[${host.hostname}:${id}] Emitted '${event.Type}:${event.Action}'`);
        emitter.host(host.hostname, events);
        const isStartEvent = status.toLowerCase() === 'start';
        const isContainer = id !== 'Host';
        if (isContainer && isStartEvent) {
            containers_1.watchContainer(host, event.id);
        }
    }
}
function parseEvent(rawEvent) {
    try {
        const event = JSON.parse(rawEvent);
        return [event];
    }
    catch (ex) {
        try {
            const events = rawEvent
                .trim()
                .split('\n')
                .map(raw => JSON.parse(raw));
            return events;
        }
        catch (ex) {
            log.warn(`Failed to parse Host Event`);
            log.debug(rawEvent.toString());
            return undefined;
        }
    }
}
//# sourceMappingURL=hosts.js.map