"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const os = require("os");
const tables = require("../");
function up(db) {
    return __awaiter(this, void 0, void 0, function* () {
        const configurationExists = yield db.schema.hasTable(tables.CONFIGURATIONS);
        if (!configurationExists) {
            yield db.schema.createTable(tables.CONFIGURATIONS, tbl => {
                tbl.text('name');
                tbl.integer('conciergePort').defaultTo(3141);
                tbl.text('proxyHostname').defaultTo('localhost');
                tbl.text('proxyIp').defaultTo('0.0.0.0');
                tbl.integer('httpPort').defaultTo(5926);
                tbl.integer('debug').defaultTo(1);
                tbl.integer('statsBinSize').defaultTo(60); // Default to 1 minute per bin
                tbl.integer('statsRetentionDays').defaultTo(1); // Truncate stats older than 1 day by default
                tbl.text('dockerRegistry').defaultTo('0.0.0.0:5000');
            });
            const defaultName = os.hostname() || 'development';
            yield db('Configurations').insert({ name: defaultName });
            log.info(`Concierge renamed to: ${defaultName}`);
        }
        const applicationsExists = yield db.schema.hasTable(tables.APPLICATIONS);
        if (!applicationsExists) {
            yield db.schema.createTable(tables.APPLICATIONS, tbl => {
                tbl.increments('id').primary();
                tbl.text('name').unique();
                tbl.text('repository').notNullable();
                tbl.text('username').defaultTo('');
                tbl.text('key').defaultTo('');
                tbl.text('dockerfile').defaultTo('');
                tbl.text('label').defaultTo('');
            });
        }
        const hostsExists = yield db.schema.hasTable(tables.HOSTS);
        if (!hostsExists) {
            yield db.schema.createTable(tables.HOSTS, tbl => {
                tbl.increments('id').primary();
                tbl.text('hostname').unique();
                tbl.text('vanityHostname').defaultTo('');
                tbl.integer('capacity').defaultTo(5);
                tbl.integer('dockerPort').defaultTo(2375);
                tbl.text('sshUsername').notNullable();
                tbl.integer('sshPort').defaultTo(22);
                tbl.text('privateKey').defaultTo('********');
            });
        }
        const conciergesExists = yield db.schema.hasTable(tables.CONCIERGES);
        if (!conciergesExists) {
            yield db.schema.createTable(tables.CONCIERGES, tbl => {
                tbl.increments('id').primary();
                tbl
                    .text('label')
                    .unique()
                    .notNullable();
                tbl.text('hostname').notNullable();
                tbl.integer('port').defaultTo(3141);
            });
        }
        const heartbeatsExists = yield db.schema.hasTable(tables.HEARTBEATS);
        if (!heartbeatsExists) {
            yield db.schema.createTable(tables.HEARTBEATS, tbl => {
                tbl.integer('hostId').notNullable();
                tbl.integer('containerId').notNullable();
                tbl.text('cpu').notNullable();
                tbl.text('memory').notNullable();
                tbl.integer('timestamp').notNullable();
            });
        }
    });
}
exports.up = up;
function down(_) {
    return __awaiter(this, void 0, void 0, function* () {
        throw new Error('Down migrations not supported');
    });
}
exports.down = down;
//# sourceMappingURL=0000_initial.js.map