"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const http = require("http");
const https = require("https");
const pkg = require('../package');
function request(url, httpModule, options) {
    options = options || { method: 'GET' };
    let urlOptions = urlToOptions(url);
    options.hostname = urlOptions.hostname;
    options.port = urlOptions.port || null;
    options.path = urlOptions.path;
    options.headers = options.headers || {};
    options.headers['User-Agent'] = options.headers['User-Agent'] || `Concierge ${pkg.version}`;
    options.asString = !options.asString ? true : options.asString;
    let promise = new Promise((resolve, reject) => {
        let body = '';
        let buffers = [];
        let httpRequest = httpModule.request(options, response => {
            response.on('end', () => {
                if (options.asString) {
                    return resolve(body);
                }
                resolve(Buffer.concat(buffers));
            });
            response.on('data', (data) => {
                if (options.asString) {
                    body += data.toString();
                    return;
                }
                buffers.push(data);
            });
        });
        httpRequest.setTimeout(options.timeout || 1500);
        httpRequest.on('timeout', () => {
            // log.warn(`${url}: Request timed out`);
            httpRequest.abort();
        });
        httpRequest.on('error', err => reject(`Request failed: ${err}`));
        httpRequest.end();
    });
    return promise;
}
function get(url, options) {
    return request(url, http, options);
}
exports.get = get;
function getHttps(url, options) {
    return request(url, https, options);
}
exports.getHttps = getHttps;
function post(url, options) {
    options = options || {};
    options.method = 'POST';
    return request(url, http, options);
}
exports.post = post;
function postHttps(url, options) {
    options = options || {};
    options.method = 'POST';
    return request(url, https, options);
}
exports.postHttps = postHttps;
function del(url, options) {
    options = options || {};
    options.method = 'DELETE';
    return request(url, http, options);
}
exports.del = del;
function delHttps(url, options) {
    options = options || {};
    options.method = 'DELETE';
    return request(url, https, options);
}
exports.delHttps = delHttps;
function put(url, options) {
    options = options || {};
    options.method = 'PUT';
    return request(url, http, options);
}
exports.put = put;
function putHttps(url, options) {
    options = options || {};
    options.method = 'PUT';
    return request(url, https, options);
}
exports.putHttps = putHttps;
function urlToOptions(url) {
    url = url.replace('http://', '');
    url = url.replace('https://', '');
    let split = url.split('/');
    let hostSplit = split[0].split(':');
    let opts = {
        hostname: hostSplit[0]
    };
    if (hostSplit[1]) {
        opts.port = hostSplit[1];
    }
    if (split.length === 1) {
        return opts;
    }
    opts.path = '/' + split.slice(1).join('/');
    return opts;
}
//# sourceMappingURL=request.js.map