"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const HTTPProxy = require("http-proxy");
const db_1 = require("../../api/configuration/db");
const find_container_1 = require("./find-container");
const proxyServer = HTTPProxy.createProxyServer({});
proxyServer.on('error', error => {
    log.error('[PROXY] ' + error);
});
proxyServer.on('proxyReq', (proxyReq, req) => {
    if (req.body && req.headers['content-type'] === 'application/json') {
        const body = typeof req.body === 'string' ? req.body : JSON.stringify(req.body);
        proxyReq.setHeader('Content-Type', 'application/json');
        proxyReq.setHeader('Content-Length', Buffer.byteLength(body));
        proxyReq.write(body);
    }
});
function webSocketHandler(request, socket, head) {
    return __awaiter(this, void 0, void 0, function* () {
        const container = yield find_container_1.default(request.headers.host);
        if (!container) {
            errorResponse(socket, 'Container not found');
            return;
        }
        const info = getProxyInfo(request.headers.host || '');
        const target = `${container.concierge.host}:${info.port}`;
        proxyServer.ws(request, socket, head, { target });
    });
}
exports.webSocketHandler = webSocketHandler;
function requestHandler(request, response, next) {
    return __awaiter(this, void 0, void 0, function* () {
        const info = getProxyInfo(request.headers.host || '');
        const config = yield db_1.getConfig();
        if (!config.proxyHostname || !info) {
            return next();
        }
        const proxyHostname = (config.proxyHostname || '').toLowerCase();
        if (proxyHostname !== info.hostname) {
            return next();
        }
        const container = yield find_container_1.default(info.name);
        if (!container) {
            errorResponse(response, 'Container not available');
            return;
        }
        const port = container.Ports.find(port => port.PrivatePort === info.port);
        if (!port) {
            errorResponse(response, 'Port not found on container');
            return;
        }
        const destHostname = container.concierge.host.proxyIp || container.concierge.host.hostname || '127.0.0.1';
        const targetUrl = `http://${destHostname}:${port.PublicPort}`;
        const options = {
            target: targetUrl
        };
        if (request.body) {
            // Do something different
            request.body = JSON.stringify(request.body);
        }
        proxyServer.web(request, response, options);
    });
}
exports.requestHandler = requestHandler;
function errorResponse(response, error) {
    response.statusCode = 503;
    response.statusMessage = 'Service unavailable: ' + error;
    response.write('Service unavailable: ' + error);
    response.end();
}
function getProxyInfo(hostname) {
    /**
     * Expected hostname format:
     * [container name]-[port].[proxy hostname]
     */
    const split = hostname.split('.');
    const idSplits = split[0].split('-');
    const name = idSplits
        .slice(0, -1)
        .join('-')
        .toLowerCase();
    const port = Number(idSplits.slice(-1)[0]);
    const proxyHostname = (split
        .slice(1)
        .join('.')
        .split(':')[0] || '').toLowerCase();
    return {
        name,
        port,
        hostname: proxyHostname
    };
}
//# sourceMappingURL=handler.js.map